# --
# Kernel/System/ProductLicense.pm - core module for product license management
# Copyright (C) 2006-2015 c.a.p.e. IT GmbH, http://www.cape-it.de
#
# written/edited by:
# * Rene(dot)Boehm(at)cape(dash)it(dot)de
#
# --
# This software comes with ABSOLUTELY NO WARRANTY. For details, see
# the enclosed file COPYING for license information (AGPL). If you
# did not receive this file, see http://www.gnu.org/licenses/agpl.txt.
# --

package Kernel::System::ProductLicense;

#use strict;
use warnings;

use MIME::Base64;

our @ObjectDependencies = (
    'Kernel::Config',
    'Kernel::System::Time',
    'Kernel::System::User',
);

our $CORE = <<COREEND;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COREEND

sub new {
    my ( $Type, %Param ) = @_;

    # allocate new hash for object
    my $Self = {%Param};
    bless( $Self, $Type );

    return $Self;
}

sub Do {
    my ( $Self, %Param ) = @_;
    
    if (!$Self->{Decoded}) {
        my $Raw = decode_base64($CORE);
        eval($Raw);
        $Self->{Decoded} = 1;
    }
        
    my $Function = $Param{Function};
    return &$Function($Self, %Param);
}

1;
